import { Component, ElementRef, HostListener, OnInit, ViewChild } from '@angular/core';
import { Observable } from 'rxjs';
import { ActivitiesService } from './activities.service';
import { animate, style, transition, trigger } from '@angular/animations';
import { ActivityModalComponent } from './activity-modal/activity-modal.component';
import { Activity, DataCard } from '@nspop/gm-web-facade-api';
import { NoteModalComponent } from './note-modal/note-modal.component';
import { CarePlanService } from '../api/care-plan.service';
import { DataCardService } from '../api/data-card.service';
import { measurementsModalEvent } from '../measurements/measurements-modal/measurements-modal.component';
import { SemiFixedHeaderComponent } from '../semi-fixed-header/semi-fixed-header.component';
import { ActivatedRoute, ParamMap } from '@angular/router';
import { take } from 'rxjs/operators';
import ActivityUtil from '../../utils/activity.util';

@Component({
  selector: 'app-activities',
  templateUrl: './activities.component.html',
  styleUrls: [ './activities.component.scss' ],
  animations: [
    trigger(
      'inOutAnimation',
      [
        transition(
          ':enter',
          [
            style({ opacity: 0 }),
            animate(
              '0.5s ease-out',
              style({ opacity: 1 })
            )
          ]
        ),
        transition(
          ':leave',
          [
            style({ opacity: 1 })
          ]
        )
      ]
    )
  ]
})
export class ActivitiesComponent implements OnInit {

  public pastActivities$: Observable<Activity[]> = this.activitiesService.pastActivities$;
  public futureActivities$: Observable<Activity[]> = this.activitiesService.futureActivities$;

  public selectedDataCard$: Observable<DataCard | undefined> = this.activitiesService.selectedDataCard$;

  public dividerPassed = false;

  @ViewChild('divider') public divider: ElementRef | undefined;
  @ViewChild('header') public header: SemiFixedHeaderComponent | undefined;
  @ViewChild('upcomingActivities') public upcomingActivities: ElementRef | undefined;
  @ViewChild('button') public button: HTMLButtonElement | undefined;

  @ViewChild('activityModal') public activityModal: ActivityModalComponent | undefined;
  @ViewChild('noteModal') public noteModal: NoteModalComponent | undefined;

  public measurementsModalEvent: measurementsModalEvent = {
    isOpen: false,
    observationType: 'regular',
    allowTypeChange: true,
    allowActivityChange: false
  };
  public saveListeners: (() => any)[] = [];
  public dataCard: DataCard | undefined;

  constructor(
    private activitiesService: ActivitiesService,
    private carePlanService: CarePlanService,
    public dataCardService: DataCardService,
    public activatedRoute: ActivatedRoute
  ) {
    this.activatedRoute.queryParamMap
      .pipe(take(1))
      .subscribe((params: ParamMap) => {
        if (!params.has('highlight')) {
          this.scrollToActivities();
        }
      });
  }

  public scrollToActivities = () => {
    const subscriber = this.activitiesService.futureActivities$.subscribe((activities: Activity[]) => {
      if (activities?.length > 0) {
        setTimeout(() => {
          const elementYPos = this.upcomingActivities?.nativeElement.getBoundingClientRect().top + window.pageYOffset;
          const floatingHeaderHeight = 160; // about 200px
          window.scrollTo({ top: elementYPos - floatingHeaderHeight, behavior: 'smooth'});
          subscriber.unsubscribe();
        }, 400);
      }
    });
  }

  ngOnInit(): void {
    this.activitiesService.openActivityModalEvent$.subscribe((activity: Activity) => {
      this.activityModal?.setActivity(activity);
      this.activityModal?.setVisible(true);
    });

    this.activitiesService.openNoteModalEvent$.subscribe((options) => {
      this.noteModal?.setNoteModalState(true, options.activity, false);
    });

    this.activitiesService.openMeasurementsModalEvent$.subscribe((activity: Activity) => {
      this.addOrEditMeasurements(activity);
    });

    this.saveListeners.push(this.saveCallback);
    this.selectedDataCard$.subscribe((dataCard?: DataCard) => {
      this.dataCard = dataCard;
    });
  }

  public addActivity(): void {
    this.activitiesService.emitOpenActivityModalEvent(undefined);
  }

  public addOrEditMeasurements(activity: Activity): void {
    if (this.measurementsModalEvent) {
      this.measurementsModalEvent.observationType = ActivityUtil.hasUltraSoundMeasurements(activity)
        ? 'ultrasound'
        : 'regular';

      this.measurementsModalEvent.isOpen = true;
      this.measurementsModalEvent.allowTypeChange = !ActivityUtil.hasMeasurements(activity);
      this.measurementsModalEvent.activity = activity;
      this.measurementsModalEvent.allowActivityChange = false;
    }
  }

  @HostListener('window:scroll', [ '$event' ]) // for window scroll events
  public onScroll(): void {
    if (this.divider && this.header) {
      const dividerPosition = this.divider.nativeElement.getBoundingClientRect().top + window.pageYOffset;
      this.dividerPassed = dividerPosition <= (window.scrollY + 200);
    }
  }

  private saveCallback = () => this.carePlanService.refresh();
}
