import { ComponentFixture, TestBed } from '@angular/core/testing';

import { MotherRelatedMeasurementsComponent } from './mother-related-measurements.component';
import {Observation} from '@nspop/gm-web-facade-api';
import {AngularMaterialModule} from '../../../modules/angular-material.module';

describe('MotherRelatedMeasurementsComponent', () => {
  let component: MotherRelatedMeasurementsComponent;
  let fixture: ComponentFixture<MotherRelatedMeasurementsComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [ MotherRelatedMeasurementsComponent ],
      imports: [AngularMaterialModule]
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(MotherRelatedMeasurementsComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('returns an observation of the right type upon getting measurement', () => {
    const observation1: Observation = {
      type: 'blood_pressure',
      values: [
        {value: '0', name: 'systolicInput', unit: '[mm/Hg]'},
        {value: '1', name: 'diastolicInput', unit: '[mm/Hg]'},
      ],
      dateOfMeasurement: new Date(),
    } as any;
    component.observations = [ observation1 ];

    expect(component.getMeasurement('blood_pressure')).toEqual(observation1);
  });

  it('returns the latest observation of the right type upon getting measurement', () => {
    const observation1: Observation = {
      type: 'blood_pressure',
      values: [
        {value: '1', name: 'systolicInput', unit: '[mm/Hg]'},
        {value: '1', name: 'diastolicInput', unit: '[mm/Hg]'},
      ],
      dateOfMeasurement: new Date('20100102'),
    } as any;

    const observation2: Observation = {
      type: 'blood_pressure',
      values: [
        {value: '2', name: 'systolicInput', unit: '[mm/Hg]'},
        {value: '2', name: 'diastolicInput', unit: '[mm/Hg]'},
      ],
      dateOfMeasurement: new Date('20100101'),
    } as any;

    component.observations = [ observation1, observation2 ];

    expect(component.getMeasurement('blood_pressure')?.values[0]?.value).toEqual('1');
  });

  it('returns empty string upon getting measurement if observations are undefined', () => {
    component.observations = undefined;

    expect(component.getMeasurement('blood_pressure')).toEqual(undefined);
  });

  it('gets the right format for bloodPressure', () => {
    const observation1: Observation = {
      type: 'blood_pressure',
      values: [
        {value: 'systolic', name: 'systolicInput', unit: 'mmHg'},
        {value: 'diastolic', name: 'diastolicInput', unit: 'mmHg'},
      ],
      dateOfMeasurement: new Date(),
    } as any;

    component.observations = [ observation1 ];

    expect(component.getBloodPressure()).toEqual('systolic / diastolic mmHg');
  });

  it('displays a placeholder if no observation was found', () => {
    component.observations = [];

    expect(component.getBloodPressure()).toEqual('');
  });

  it('displays a placeholder if no systolic value was found', () => {
    const observation1: Observation = {
      type: 'blood_pressure',
      values: [
      ],
      dateOfMeasurement: new Date(),
    } as any;

    component.observations = [ observation1 ];

    expect(component.getBloodPressure()).toEqual(' /  ');
  });

  it('displays a placeholder if no systolic value was found', () => {
    const observation1: Observation = {
      type: 'blood_pressure',
      values: [
        {value: 'systolic', name: 'systolicInput', unit: 'mmHg'},
      ],
      dateOfMeasurement: new Date(),
    } as any;

    component.observations = [ observation1 ];

    expect(component.getBloodPressure()).toEqual('systolic /  mmHg');
  });

  it('displays the right value when getting single measurement', () => {
    const observation1: Observation = {
      type: 'weight',
      values: [
        {value: '50', name: '', unit: 'kg'},
      ],
      dateOfMeasurement: new Date(),
    } as any;

    component.observations = [ observation1 ];

    expect(component.getSingleMeasurementValue('weight')).toEqual('50 kg');
  });

  it('displays a placeholder if no observation found when getting single measurement', () => {
    component.observations = [ ];

    expect(component.getSingleMeasurementValue('weight')).toEqual('');
  });

  it('displays nothing if no value or unit found when getting single measurement', () => {
    const observation1: Observation = {
      type: 'weight',
      values: [
        {value: undefined, name: '', unit: undefined},
      ],
      dateOfMeasurement: new Date(),
    } as any;

    component.observations = [ observation1 ];

    expect(component.getSingleMeasurementValue('weight')).toEqual(' ');
  });

  it('should be able to get multiple measurments values', () => {
    const observation1: Observation = {
      type: 'blood_pressure',
      values: [
        {value: 75, name: 'systolic', unit: 'foo'},
        {value: 30, name: 'diastolic', unit: 'bar'},
      ],
      dateOfMeasurement: new Date(),
    } as any;

    component.observations = [ observation1 ];
    const measurementValues = component.getMultipleMeasurementValues('blood_pressure', true);
    expect(measurementValues).toBe('75 foo, 30 bar');
  });

  it('should return an empty string if there are no measuremenets', () => {
    const observation1: Observation = {
      type: 'blood_pressure'
    } as any;

    component.observations = [ observation1 ];
    const measurementValues = component.getMultipleMeasurementValues('blood_pressure');
    expect(measurementValues).toBe('');
  });
});
