import {Component, EventEmitter, Input, OnChanges, OnInit, Output, SimpleChanges, ViewChild} from '@angular/core';
import {ModalComponent} from '../../modals/modal/modal.component';
import {
  Activity,
  DataCard,
  Note,
  ObservationTable,
  ObservationTableFetuses,
  ObservationTableMotherRows,
  ObservationTableRows,
  SaveRegularObservationsRequest,
  SaveRegularObservationsRequestBloodPressure,
  SaveRegularObservationsRequestFetuses,
  SaveUltrasoundObservationsRequest,
  SaveUltrasoundObservationsRequestFetuses,
  ScanningObservationTableFetuses,
  ScanningObservationTableMotherRows,
  ScanningObservationTableRows,
} from '@nspop/gm-web-facade-api';
import {FormComponent} from '../../confirm-navigation-if-dirty.framework';
import {NgForm} from '@angular/forms';
import {copy} from '../../../utils/copy';
import {combineLatest, Observable, of} from 'rxjs';
import {concatMap, map} from 'rxjs/operators';
import {DataCardWrapper} from '../../model-wrappers/dataCard.wrapper';
import {ObservationService} from '../../api/observation.service';
import {DataCardService} from '../../api/data-card.service';
import {NoteService} from '../../api/note.service';
import FlowEnum = SaveUltrasoundObservationsRequest.FlowEnum;
import {AppError} from '../../services/error.service';

export type measurementsModalEvent = {
  isOpen: boolean;
  observationType: ObservationFormType;
  allowTypeChange: boolean;
  activity?: Activity;
  allowActivityChange: boolean;
};

export type ObservationFormType = 'regular' | 'ultrasound';
@Component({
  selector: 'app-measurements-modal',
  templateUrl: './measurements-modal.component.html',
  styleUrls: ['./measurements-modal.component.scss']
})
export class MeasurementsModalComponent extends FormComponent implements OnChanges, OnInit {

  constructor(
    private dataCardService: DataCardService,
    private observationService: ObservationService,
    private noteService: NoteService
  ) {
    super();
  }
  public note: Note = {
    content: ''
  };

  public isSaving = false;
  public dataCard?: DataCardWrapper;
  public observationDataRegular?: SaveRegularObservationsRequest | undefined;
  public observationDataUltrasound?: SaveUltrasoundObservationsRequest | undefined;
  public measurementDate: Date | undefined;

  @ViewChild('modal') public modalComponent: ModalComponent | undefined;
  @ViewChild('form') public form: NgForm | undefined;

  @Input() public isOpen = false;
  @Input() public allowTypeChange = true;
  @Input() public activity: Activity | undefined = undefined;
  @Input() public allowActivityChange = true;
  @Input() public observationType: ObservationFormType = 'regular';

  @Output() public modalClosed: EventEmitter<void> = new EventEmitter();
  @Output() public save: EventEmitter<void> = new EventEmitter();

  private dataCardId = '';
  private dataCardVersion = '';
  private ultrasoundActivityKeywords: string[] = ['scanning', 'skanning', 'ultralyd', 'ultralydsundersøgelse', 'sonograf'];

  public static getRelevantScanningFetusRows(activityId: string | undefined, fetuses: Array<ScanningObservationTableFetuses> | undefined): any[] {
    if (!fetuses || !activityId) {
      return [];
    }
    return fetuses.filter((fetus: ScanningObservationTableFetuses) => {
      return fetus.rows && fetus.rows?.filter((row: ScanningObservationTableRows) => row.activityId === activityId ).length > 0;
    }).map((fetus: ScanningObservationTableFetuses) => {
      if (fetus.rows) {
        return {
          id: fetus.id,
          fetusPresentation: fetus.rows[0].fetusPresentation,
          ulWeight: fetus.rows[0].ulWeight?.value,
          weightDeviation: fetus.rows[0].weightDeviation?.value
        };
      }
    });
  }

  public static getRelevantObservationFetusRows(activityId: string | undefined, fetuses: Array<ObservationTableRows> | undefined): any[] {
    if (!fetuses || !activityId) {
      return [];
    }
    return fetuses.filter((fetus: ObservationTableFetuses) => {
      return fetus.rows && fetus.rows?.filter((row: ScanningObservationTableRows) => row.activityId === activityId ).length > 0;
    }).map((fetus: ObservationTableFetuses) => {
      if (fetus.rows) {
        return {
          id: fetus.id,
          fetusPresentation: fetus.rows[0].fetusPresentation,
          fetusWeightEstimate: fetus.rows[0].fetusWeightEstimate?.value,
          fetusActivity: fetus.rows[0].fetusActivity
        };
      }
    });
  }

  ngOnInit(): void {
    this.dataCardService.selectedDataCard$.subscribe((dataCard?: DataCard) => {
      if (dataCard) {
        this.dataCardId = dataCard.documentUniqueId || '';
        this.dataCardVersion = dataCard.version || '';
        this.dataCard = new DataCardWrapper(copy(dataCard));
      }
    });
  }

  ngOnChanges(changes: SimpleChanges): void {
    if (changes.isOpen) {
      if (changes.isOpen.currentValue) {
        this.initObservationData();
        this.modalComponent?.setVisible(true);
      } else {
        this.handleClose();
      }
    }
    if (this.isOpen && changes.observationType) {
      this.initObservationData();
    }
  }

  public handleClose(): void {
    if (!this.isOpen) {
      return;
    }

    this.modalComponent?.setVisible(false);

    if (this.modalComponent?.isModalVisible) {
      return;
    }
    this.modalClosed.emit();

    this.clearObservationData();
  }

  public activityChanged(): void {
    const activitySelector = document.getElementById('activitySelector');
    this.setFormValues(this.activity);
    if (activitySelector) {
      if (!this.activity && this.modalComponent && this.modalComponent.isModalVisible) {
        activitySelector.classList.add('ng-invalid');
      }
      activitySelector.classList.remove('ng-invalid');
    }
  }

  public setFormValues(activity: Activity | undefined): void {
    if (!activity) {
      return;
    }

    function getDateFromActivity(selectedActivity: Activity | undefined): Date | undefined {
      if (!selectedActivity) {
        return;
      }
      switch (selectedActivity.status) {
        case 'finished':
          if (selectedActivity.conductedTime) {
            return selectedActivity.conductedTime;
          } else {
            return selectedActivity.plannedTime?.from;
          }
        default:
          return new Date();
      }
    }

    this.note = activity?.note ?? { content: ''};
    this.measurementDate = getDateFromActivity(activity);
  }

  public doSave(): void {
    if (this.form?.valid) {
      if (!this.activity) {
        const activitySelector = document.getElementById('activitySelector');
        activitySelector?.scrollIntoView({behavior: 'smooth', block: 'center'});
        return;
      }
      const activityId = this.activity?.id;
      this.setMeasurementDate(this.measurementDate);
      this.setActivityId(activityId);
      this.isSaving = true;
      const save = (this.observationType === 'ultrasound') ? this.saveUltrasound() : this.saveRegular();
      save.pipe(
        concatMap(() => this.noteService.createOrUpdateNote(activityId, this.note, this.note.date)))
        .subscribe(() => {
        }, (error => {
          throw new AppError(error?.errorMessage, error);
        }), async () => {
          await this.dataCardService.refresh();
          this.isSaving = false;
          this.handleClose();
        });
    } else {
      this.form?.resetForm(this.form?.form.value); // We make a fake reset, so the form is not seen as 'submitted'
      this.form?.form.markAsDirty(); // (Ensuring we still get prompted that we are going to discard changes if we try to leave the page)
      return;
    }
  }

  private saveUltrasound(): Observable<void> {
    if (this.dataCard) {
      return this.dataCardService.update(this.dataCard.asDataCard, this.dataCardId, this.dataCardVersion)
        .pipe(
          concatMap(
            () => this.observationService.createUltrasoundObservations(this.observationDataUltrasound as SaveUltrasoundObservationsRequest))
        ) as unknown as Observable<void>;
    }
    return of();
  }

  private saveRegular(): Observable<void> {
    return this.observationService.createRegularObservations(this.observationDataRegular as SaveRegularObservationsRequest);
  }

  private setActivityId(activityId?: string): void {
    if (!this.observationDataRegular || !this.observationDataUltrasound) {
      return;
    }
    this.observationDataRegular.activityId = activityId;
    this.observationDataUltrasound.activityId = activityId;
  }

  private setMeasurementDate(measurementDate?: Date): void {
    if (!measurementDate) {
      return;
    }

    const offsetDate = this.dataCard?.expectedDateOfDeliverySection?.offsetDate;
    this.setMeasurementDateRegular(measurementDate, offsetDate);
    this.setMeasurementDateUltrasound(measurementDate, offsetDate);
  }

  private setMeasurementDateRegular(measurementDate: Date, offsetDate?: Date): void {
    if (!this.observationDataRegular) {
      return;
    }

    this.observationDataRegular.date = measurementDate;
    this.observationDataRegular.offsetDate = offsetDate;
  }

  private setMeasurementDateUltrasound(measurementDate: Date, offsetDate?: Date): void {
    if (!this.observationDataUltrasound) {
      return;
    }

    this.observationDataUltrasound.date = measurementDate;
    this.observationDataUltrasound.offsetDate = offsetDate;
  }

  public async initObservationData(): Promise<void> {
    if (this.activity?.observations?.length) {
      this.setMeasurementEditModel();
    } else {
      this.setTypeChangeAllowed(true);
      this.observationDataRegular = {
        fetuses: [...new Array(this.dataCard?.fetusCount || 1)].map((x, index) => ({id: this.getNthLetter(index)})),
      };
      this.observationDataUltrasound = {
        fetuses: [...new Array(this.dataCard?.fetusCount || 1)].map((x, index) => ({id: this.getNthLetter(index)})),
      };
      const title = this.activity?.title ? this.activity.title : '';
      this.observationType = this.isUltrasoundFromTitle(title) ? 'ultrasound' : this.observationType;
    }

    this.observationDataRegular = copy(this.observationDataRegular);
    this.observationDataUltrasound = copy(this.observationDataUltrasound);

    this.setFormValues(this.activity);


    this.form?.form.markAsPristine();
    this.form?.form.markAsUntouched();
  }

  private setMeasurementEditModel(): void {
    function tableRowMatchesActivity(id: string | undefined, table: ObservationTable | undefined): boolean {
      if (!table || !id) {
        return false;
      }
      const hasMatch = table.mother?.rows?.some((row: ScanningObservationTableMotherRows) => row.activityId === id) ||
        table.fetuses?.some(fetus => fetus?.rows?.some(row => row.activityId === id));
      return hasMatch !== undefined ? hasMatch : false;
    }

    function mapFlowValue(flow: string): FlowEnum {
      switch (flow?.toLowerCase()) {
        case 'normal':
          return 'normal';
        case 'afvigende':
          return 'abnormal';
        case 'der følges op':
          return 'tobecontinued';
        default:
          return 'normal';
      }
    }

    function mapToBloodPressure(bloodPressure: string | undefined): SaveRegularObservationsRequestBloodPressure | undefined {
      const bp = bloodPressure?.split('/');
      if (bp) {
        return {
          systolic: Number.parseInt(bp[0], 10),
          diastolic: Number.parseInt(bp[1], 10)
        };
      }
    }
    const offsetDate = this.dataCard?.expectedDateOfDeliverySection?.offsetDate;
    combineLatest([this.observationService.ultrasoundObservationTable$, this.observationService.clinicalObservationTable$]).pipe(
      map(([uSoundTable, clinicTable]) => {
        if (this.activity) {
          if (tableRowMatchesActivity(this.activity.id, uSoundTable)) {
            this.observationType = 'ultrasound';
            this.setTypeChangeAllowed(false);
            const motherRows = uSoundTable?.mother?.rows?.find((row: ScanningObservationTableMotherRows) => row.activityId === this.activity?.id);
            const fetusRows: SaveUltrasoundObservationsRequestFetuses[] = MeasurementsModalComponent.getRelevantScanningFetusRows(this.activity?.id, uSoundTable?.fetuses);
            this.observationDataUltrasound = {
              activityId: this.activity?.id,
              documentUniqueId: motherRows?.id || fetusRows[0]?.id,
              amnioticFluid: motherRows?.amnioticFluid?.value,
              cervix: motherRows?.cervix?.value,
              dvp: motherRows?.dvp?.value,
              fetuses: fetusRows
            };
            if (motherRows?.flow) {
              this.observationDataUltrasound.flow = mapFlowValue(motherRows.flow);
            }
            if (offsetDate) {
              this.observationDataUltrasound.offsetDate = offsetDate;
            }
            if (!this.observationDataUltrasound?.fetuses || this.observationDataUltrasound?.fetuses?.length <= 0) {
              this.observationDataUltrasound
                .fetuses = [...new Array(this.dataCard?.fetusCount || 1)].map((x, index) => ({id: this.getNthLetter(index)}));
            }
          } else if (tableRowMatchesActivity(this.activity?.id, clinicTable)) {
            const title = this.activity?.title ? this.activity.title : '';
            this.observationType = this.isUltrasoundFromTitle(title) ? 'ultrasound' : this.observationType;
            this.setTypeChangeAllowed(false);
            const motherRows: ObservationTableMotherRows | undefined = clinicTable?.mother?.rows?.find((row: ObservationTableMotherRows) => row.activityId === this.activity?.id);
            const fetusRows: SaveRegularObservationsRequestFetuses[] = MeasurementsModalComponent.getRelevantObservationFetusRows(this.activity?.id, clinicTable?.fetuses);
            this.observationDataRegular = {
              activityId: this.activity?.id,
              pubismeasurement: motherRows?.pubismeasurement?.value,
              weight: motherRows?.weight?.value,
              bloodPressure: mapToBloodPressure(motherRows?.bloodPressure),
              edema: motherRows?.edema,
              urine: motherRows?.urine ? motherRows.urine.split(', ') : [],
              documentUniqueId: motherRows?.id || fetusRows[0]?.id,
              fetuses: fetusRows
            };
            if (offsetDate) {
              this.observationDataRegular.offsetDate = offsetDate;
            }
            if (!this.observationDataRegular?.fetuses || this.observationDataRegular?.fetuses?.length <= 0) {
              this.observationDataRegular
                .fetuses = [...new Array(this.dataCard?.fetusCount || 1)].map((x, index) => ({id: this.getNthLetter(index)}));
            }
          }
        }
      })
    ).subscribe();
  }

  public isObservationDataInitialized(): boolean {
    return !!this.observationDataRegular && !!this.observationDataUltrasound;
  }

  public getNthLetter(position: number): string {
    return String.fromCharCode('ABCDEFGHIJ'.charCodeAt(position));
  }

  private clearObservationData(): void {
    this.activity = undefined;
    this.observationDataRegular = undefined;
    this.observationDataUltrasound = undefined;
    this.measurementDate = undefined;
  }

  public isDatePickerDisabled(activity: Activity): boolean {
    if (!activity) {
      return true;
    }

    return activity.status === 'finished';
  }

  public isEditing(): boolean {
    if (!this.activity?.observations) {
      return false;
    }
    return this.activity.observations.length > 0;
  }

  public canChangeObservationType(): boolean {
    const isDirty = this.form && this.form?.dirty !== null && this.form.dirty;
    return this.getTypeChangeAllowed() && !isDirty;
  }


  private setTypeChangeAllowed(isAllowed: boolean): void {
    this.allowTypeChange = isAllowed;
  }

  private getTypeChangeAllowed(): boolean {
    return this.allowTypeChange;
  }

  private isUltrasoundFromTitle(title: string): boolean {
    return title.includes('UL')
        || this.ultrasoundActivityKeywords.some(keyword => title.toLowerCase().includes(keyword));
  }
}
