import { ComponentFixture, TestBed } from '@angular/core/testing';

import { BloodPressureFormFieldComponent } from './blood-pressure-form-field.component';
import {SimpleChange, SimpleChanges} from '@angular/core';
import {By} from '@angular/platform-browser';
import {AbstractControl} from '@angular/forms';

describe('BloodPressureFormFieldComponent', () => {
  let component: BloodPressureFormFieldComponent;
  let fixture: ComponentFixture<BloodPressureFormFieldComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [ BloodPressureFormFieldComponent ]
    })
      .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(BloodPressureFormFieldComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('updates inner value when ngModel is updated on the outside', () => {
    const val = {systolic: 1, diastolic: 2};
    component.writeValue(val);
    expect(component.innerModel).toEqual(val);
  });

  it('makes ngModel undefined when set to not relevant', () => {
    const val = {systolic: 1, diastolic: 2};
    component.writeValue(val);
    fixture.detectChanges();
    component.notRelevant = true;
    component.ngOnChanges({notRelevant: {currentValue: true, previousValue: undefined} as SimpleChange} as SimpleChanges);
    expect(component.value).toBe(undefined);
  });

  it('re-fills the input when set to not relevant and back again', () => {
    const val = {systolic: 1, diastolic: 2};
    component.writeValue(val);
    fixture.detectChanges();
    component.notRelevant = true;
    component.ngOnChanges({notRelevant: {currentValue: true, previousValue: undefined} as SimpleChange} as SimpleChanges);
    expect(component.value).toBe(undefined);
    component.notRelevant = false;
    component.ngOnChanges({notRelevant: {currentValue: false, previousValue: true} as SimpleChange} as SimpleChanges);
    expect(component.value).toBe(val);
  });

  it('does not register a change on the first value set', () => {
    // This is to avoid issues where values initialized from an async call will mark the form as dirty
    const mock = {
      onChange: (whatever: any) => undefined
    };

    const val = {systolic: 1, diastolic: 2};
    const onChange = spyOn(mock, 'onChange');
    component.registerOnChange(onChange);
    component.writeValue(val);
    expect(onChange).not.toHaveBeenCalled();

    fixture.debugElement.query(By.css('.input')).triggerEventHandler('focus', {});
    const val2 =  {systolic: 10, diastolic: 20};
    component.writeValue(val2);
    expect(onChange).toHaveBeenCalledWith(val2);
  });

  it('becomes dirtyable when clicked', () => {
    fixture.debugElement.query(By.css('.input')).triggerEventHandler('focus', {});
    fixture.detectChanges();
    expect((component as any).dirtyAble).toBeTrue();
  });

  it('should invoke some callback functions', () => {
    const touchedCallbackFn = () => 'touched';
    component.registerOnTouched(touchedCallbackFn);
    const changedCallbackFn = () => 'changed';
    component.registerOnChange(changedCallbackFn);
  });

  it('should allow setting itself as disabled', () => {
    expect(component.disabled).toBeFalse();
    component.setDisabledState(true);
    expect(component.disabled).toBeTrue();
  });

  it('should trigger touched on blur', () => {
    const callbacks = {
      touchedCallbackFn: () => 'touched'
    };
    const spy = spyOn(callbacks, 'touchedCallbackFn').and.callThrough();
    component.registerOnTouched(callbacks.touchedCallbackFn);
    component.onBlur();
    expect(spy).toHaveBeenCalledTimes(1);
  });

  it('should validate the bloodpressure input', () => {
    let validate = component.validate({} as AbstractControl);
    expect(validate).toBeNull();
    component.bloodPressureInput = {
      errorState: true
    } as any;
    validate = component.validate({} as AbstractControl);
    expect(validate).toEqual({
      onlyOneFieldIsFilled: 'Only one value is filled. For blood pressure both systolic and diastolic should be filled.'
    });
  });
});
